@@ -21,9 +21,6 @@
 #include "ges.h"
 #include "ges-internal.h"
 
-/* TODO:
- *  + Handle Groups
- **/
 #define parent_class ges_base_xml_formatter_parent_class
 G_DEFINE_ABSTRACT_TYPE (GESBaseXmlFormatter, ges_base_xml_formatter,
     GES_TYPE_FORMATTER);
@@ -51,6 +48,19 @@ typedef struct PendingBinding
   gchar *binding_type;
 } PendingBinding;
 
+typedef struct PendingChildProperties
+{
+  gchar *track_id;
+  GstStructure *structure;
+} PendingChildProperties;
+
+typedef struct PendingGroup
+{
+  GESGroup *group;
+
+  GList *pending_children;
+} PendingGroup;
+
 typedef struct PendingClip
 {
   gchar *id;
@@ -69,6 +79,8 @@ typedef struct PendingClip
 
   GList *pending_bindings;
 
+  GList *children_props;
+
   /* TODO Implement asset effect management
    * PendingTrackElements *track_elements; */
 } PendingClip;
@@ -98,7 +110,7 @@ struct _GESBaseXmlFormatterPrivate
   GHashTable *clipid_pendings;
 
   /* Clip.ID -> Clip */
-  GHashTable *clips;
+  GHashTable *containers;
 
   /* ID -> track */
   GHashTable *tracks;
@@ -116,6 +128,8 @@ struct _GESBaseXmlFormatterPrivate
   PendingClip *current_pending_clip;
 
   gboolean timeline_auto_transition;
+
+  GList *groups;
 };
 
 static void
@@ -170,6 +184,9 @@ create_parser_context (GESBaseXmlFormatter * self, const gchar * uri,
           &err) == FALSE)
     goto failed;
 
+  if (!g_markup_parse_context_end_parse (parsecontext, &err))
+    goto failed;
+
 done:
   if (xmlcontent)
     g_free (xmlcontent);
@@ -325,7 +342,7 @@ _dispose (GObject * object)
 
   g_clear_pointer (&priv->assetid_pendingclips,
       (GDestroyNotify) g_hash_table_unref);
-  g_clear_pointer (&priv->clips, (GDestroyNotify) g_hash_table_unref);
+  g_clear_pointer (&priv->containers, (GDestroyNotify) g_hash_table_unref);
   g_clear_pointer (&priv->clipid_pendings, (GDestroyNotify) g_hash_table_unref);
   g_clear_pointer (&priv->tracks, (GDestroyNotify) g_hash_table_unref);
   g_clear_pointer (&priv->layers, (GDestroyNotify) g_hash_table_unref);
@@ -363,7 +380,7 @@ ges_base_xml_formatter_init (GESBaseXmlFormatter * self)
       g_str_equal, g_free, NULL);
   priv->clipid_pendings = g_hash_table_new_full (g_str_hash,
       g_str_equal, g_free, NULL);
-  priv->clips = g_hash_table_new_full (g_str_hash,
+  priv->containers = g_hash_table_new_full (g_str_hash,
       g_str_equal, g_free, gst_object_unref);
   priv->tracks = g_hash_table_new_full (g_str_hash,
       g_str_equal, g_free, gst_object_unref);
@@ -414,11 +431,38 @@ _set_auto_transition (gpointer prio, LayerEntry * entry, gpointer udata)
   ges_layer_set_auto_transition (entry->layer, entry->auto_trans);
 }
 
+static void
+_add_all_groups (GESFormatter * self)
+{
+  GList *tmp;
+  GESTimelineElement *child;
+  GESBaseXmlFormatterPrivate *priv = GES_BASE_XML_FORMATTER (self)->priv;
+
+  for (tmp = priv->groups; tmp; tmp = tmp->next) {
+    GList *lchild;
+    PendingGroup *pgroup = tmp->data;
+
+    for (lchild = ((PendingGroup *) tmp->data)->pending_children; lchild;
+        lchild = lchild->next) {
+      child = g_hash_table_lookup (priv->containers, lchild->data);
+
+      GST_DEBUG_OBJECT (tmp->data, "Adding %s child %" GST_PTR_FORMAT " %s",
+          (const gchar *) lchild->data, child,
+          GES_TIMELINE_ELEMENT_NAME (child));
+      ges_timeline_element_set_timeline (GES_TIMELINE_ELEMENT (pgroup->group),
+          self->timeline);
+      ges_container_add (GES_CONTAINER (pgroup->group), child);
+    }
+  }
+}
+
 static void
 _loading_done (GESFormatter * self)
 {
   GESBaseXmlFormatterPrivate *priv = GES_BASE_XML_FORMATTER (self)->priv;
 
+  _add_all_groups (self);
+
   if (priv->parsecontext)
     g_markup_parse_context_free (priv->parsecontext);
   priv->parsecontext = NULL;
@@ -439,26 +483,29 @@ _loading_done_cb (GESFormatter * self)
   return FALSE;
 }
 
-static void
+static gboolean
 _set_child_property (GQuark field_id, const GValue * value,
     GESTrackElement * effect)
 {
   GParamSpec *pspec;
   GstElement *element;
 
+  /* FIXME: error handling? */
   if (!ges_track_element_lookup_child (effect,
           g_quark_to_string (field_id), &element, &pspec))
-    return;
+    return TRUE;
 
   g_object_set_property (G_OBJECT (element), pspec->name, value);
   g_param_spec_unref (pspec);
   gst_object_unref (element);
+  return TRUE;
 }
 
-void
+gboolean
 set_property_foreach (GQuark field_id, const GValue * value, GObject * object)
 {
   g_object_set_property (object, g_quark_to_string (field_id), value);
+  return TRUE;
 }
 
 static inline GESClip *
@@ -486,7 +533,7 @@ _add_object_to_layer (GESBaseXmlFormatterPrivate * priv, const gchar * id,
     gst_structure_foreach (properties,
         (GstStructureForeachFunc) set_property_foreach, clip);
 
-  g_hash_table_insert (priv->clips, g_strdup (id), gst_object_ref (clip));
+  g_hash_table_insert (priv->containers, g_strdup (id), gst_object_ref (clip));
   return clip;
 }
 
@@ -511,6 +558,18 @@ _add_track_element (GESFormatter * self, GESClip * clip,
   ges_container_add (GES_CONTAINER (clip), GES_TIMELINE_ELEMENT (trackelement));
   gst_structure_foreach (children_properties,
       (GstStructureForeachFunc) _set_child_property, trackelement);
+
+  if (properties)
+    gst_structure_foreach (properties,
+        (GstStructureForeachFunc) set_property_foreach, trackelement);
+}
+
+static void
+_free_pending_children_props (PendingChildProperties * pend)
+{
+  g_free (pend->track_id);
+  if (pend->structure)
+    gst_structure_free (pend->structure);
 }
 
 static void
@@ -543,6 +602,8 @@ _free_pending_clip (GESBaseXmlFormatterPrivate * priv, PendingClip * pend)
   g_list_free_full (pend->effects, (GDestroyNotify) _free_pending_effect);
   g_list_free_full (pend->pending_bindings,
       (GDestroyNotify) _free_pending_binding);
+  g_list_free_full (pend->children_props,
+      (GDestroyNotify) _free_pending_children_props);
   g_hash_table_remove (priv->clipid_pendings, pend->id);
   g_free (pend->id);
   g_slice_free (PendingClip, pend);
@@ -560,6 +621,23 @@ _free_pending_asset (GESBaseXmlFormatterPrivate * priv, PendingAsset * passet)
   g_slice_free (PendingAsset, passet);
 }
 
+static void
+_add_children_properties (GESBaseXmlFormatterPrivate * priv, GList * childprops,
+    GESClip * clip)
+{
+  GList *tmpchildprops;
+
+  for (tmpchildprops = childprops; tmpchildprops;
+      tmpchildprops = tmpchildprops->next) {
+    PendingChildProperties *pchildprops = tmpchildprops->data;
+    GESTrackElement *element =
+        _get_element_by_track_id (priv, pchildprops->track_id, clip);
+    if (element && pchildprops->structure)
+      gst_structure_foreach (pchildprops->structure,
+          (GstStructureForeachFunc) _set_child_property, element);
+  }
+}
+
 static void
 _add_pending_bindings (GESBaseXmlFormatterPrivate * priv, GList * bindings,
     GESClip * clip)
@@ -650,6 +728,7 @@ new_asset_cb (GESAsset * source, GAsyncResult * res, PendingAsset * passet)
     if (clip == NULL)
       continue;
 
+    _add_children_properties (priv, pend->children_props, clip);
     _add_pending_bindings (priv, pend->pending_bindings, clip);
 
     GST_DEBUG_OBJECT (self, "Adding %i effect to new object",
@@ -691,7 +770,7 @@ _create_profile (GESBaseXmlFormatter * self,
     const gchar * type, const gchar * parent, const gchar * name,
     const gchar * description, GstCaps * format, const gchar * preset,
     const gchar * preset_name, gint id, guint presence, GstCaps * restriction,
-    guint pass, gboolean variableframerate)
+    guint pass, gboolean variableframerate, gboolean enabled)
 {
   GstEncodingProfile *profile = NULL;
 
@@ -719,6 +798,7 @@ _create_profile (GESBaseXmlFormatter * self,
   }
 
   gst_encoding_profile_set_name (profile, name);
+  gst_encoding_profile_set_enabled (profile, enabled);
   gst_encoding_profile_set_description (profile, description);
   gst_encoding_profile_set_preset_name (profile, preset_name);
 
@@ -942,7 +1022,8 @@ ges_base_xml_formatter_add_track (GESBaseXmlFormatter * self,
 
     gst_structure_get (properties, "restriction-caps", G_TYPE_STRING,
         &restriction, NULL);
-    gst_structure_remove_fields (properties, "restriction-caps", "caps", NULL);
+    gst_structure_remove_fields (properties, "restriction-caps", "caps",
+        "message-forward", NULL);
     if (g_strcmp0 (restriction, "NULL")) {
       caps = gst_caps_from_string (restriction);
       ges_track_set_restriction_caps (track, caps);
@@ -1009,6 +1090,40 @@ ges_base_xml_formatter_add_control_binding (GESBaseXmlFormatter * self,
     GST_WARNING ("This interpolation type is not supported\n");
 }
 
+void
+ges_base_xml_formatter_add_source (GESBaseXmlFormatter * self,
+    const gchar * track_id, GstStructure * children_properties)
+{
+  GESBaseXmlFormatterPrivate *priv = _GET_PRIV (self);
+  GESTrackElement *element = NULL;
+
+  if (track_id[0] != '-' && priv->current_clip)
+    element = _get_element_by_track_id (priv, track_id, priv->current_clip);
+
+  else if (track_id[0] != '-' && priv->current_pending_clip) {
+    PendingChildProperties *pchildprops;
+
+    pchildprops = g_slice_new0 (PendingChildProperties);
+    pchildprops->track_id = g_strdup (track_id);
+    pchildprops->structure = children_properties ?
+        gst_structure_copy (children_properties) : NULL;
+    priv->current_pending_clip->children_props =
+        g_list_append (priv->current_pending_clip->children_props, pchildprops);
+    return;
+  } else {
+    element = priv->current_track_element;
+  }
+
+  if (element == NULL) {
+    GST_WARNING
+        ("No current track element to which we can append children properties");
+    return;
+  }
+
+  gst_structure_foreach (children_properties,
+      (GstStructureForeachFunc) _set_child_property, element);
+}
+
 void
 ges_base_xml_formatter_add_track_element (GESBaseXmlFormatter * self,
     GType track_element_type, const gchar * asset_id, const gchar * track_id,
@@ -1052,7 +1167,7 @@ ges_base_xml_formatter_add_track_element (GESBaseXmlFormatter * self,
       ges_meta_container_add_metas_from_string (GES_META_CONTAINER
           (trackelement), metadatas);
 
-    clip = g_hash_table_lookup (priv->clips, timeline_obj_id);
+    clip = g_hash_table_lookup (priv->containers, timeline_obj_id);
     if (clip) {
       _add_track_element (GES_FORMATTER (self), clip, trackelement, track_id,
           children_properties, properties);
@@ -1096,7 +1211,7 @@ ges_base_xml_formatter_add_encoding_profile (GESBaseXmlFormatter * self,
     const gchar * description, GstCaps * format, const gchar * preset,
     const gchar * preset_name, guint id, guint presence, GstCaps * restriction,
     guint pass, gboolean variableframerate, GstStructure * properties,
-    GError ** error)
+    gboolean enabled, GError ** error)
 {
   const GList *tmp;
   GstEncodingProfile *profile;
@@ -1109,7 +1224,8 @@ ges_base_xml_formatter_add_encoding_profile (GESBaseXmlFormatter * self,
   if (parent == NULL) {
     profile =
         _create_profile (self, type, parent, name, description, format, preset,
-        preset_name, id, presence, restriction, pass, variableframerate);
+        preset_name, id, presence, restriction, pass, variableframerate,
+        enabled);
     ges_project_add_encoding_profile (GES_FORMATTER (self)->project, profile);
     gst_object_unref (profile);
 
@@ -1142,7 +1258,7 @@ ges_base_xml_formatter_add_encoding_profile (GESBaseXmlFormatter * self,
 
   profile =
       _create_profile (self, type, parent, name, description, format, preset,
-      preset_name, id, presence, restriction, pass, variableframerate);
+      preset_name, id, presence, restriction, pass, variableframerate, enabled);
 
   if (profile == NULL)
     goto done;
@@ -1155,3 +1271,44 @@ done:
   if (restriction)
     gst_caps_unref (restriction);
 }
+
+void
+ges_base_xml_formatter_add_group (GESBaseXmlFormatter * self,
+    const gchar * id, const gchar * properties)
+{
+  PendingGroup *pgroup;
+  GESBaseXmlFormatterPrivate *priv = _GET_PRIV (self);
+
+  if (priv->check_only)
+    return;
+
+  pgroup = g_slice_new0 (PendingGroup);
+  pgroup->group = ges_group_new ();
+
+  g_hash_table_insert (priv->containers, g_strdup (id),
+      gst_object_ref (pgroup->group));
+  priv->groups = g_list_prepend (priv->groups, pgroup);
+
+  return;
+}
+
+void
+ges_base_xml_formatter_last_group_add_child (GESBaseXmlFormatter * self,
+    const gchar * child_id, const gchar * name)
+{
+  PendingGroup *pgroup;
+  GESBaseXmlFormatterPrivate *priv = _GET_PRIV (self);
+
+  if (priv->check_only)
+    return;
+
+  g_return_if_fail (priv->groups);
+
+  pgroup = priv->groups->data;
+
+  pgroup->pending_children =
+      g_list_prepend (pgroup->pending_children, g_strdup (child_id));
+
+  GST_DEBUG_OBJECT (self, "Adding %s to %s", child_id,
+      GES_TIMELINE_ELEMENT_NAME (((PendingGroup *) priv->groups->data)->group));
+}
