@@ -46,6 +46,9 @@
 #include "ges.h"
 #include "ges-internal.h"
 
+static GPtrArray *new_paths = NULL;
+static GHashTable *tried_uris = NULL;
+
 /* TODO We should rely on both extractable_type and @id to identify
  * a Asset, not only @id
  */
@@ -98,7 +101,6 @@ static GParamSpec *_properties[LAST_SIGNAL] = { 0 };
 static gboolean
 _emit_loaded_in_idle (EmitLoadedInIdle * data)
 {
-  ges_timeline_commit (data->timeline);
   g_signal_emit (data->project, _signals[LOADED_SIGNAL], 0, data->timeline);
 
   gst_object_unref (data->project);
@@ -149,8 +151,8 @@ ges_project_set_uri (GESProject * project, const gchar * uri)
     return;
   }
 
-  if (uri == NULL || !gst_uri_is_valid (uri)) {
-    GST_LOG_OBJECT (project, "Invalid URI: %s", uri);
+  if (uri == NULL) {
+    GST_LOG_OBJECT (project, "Uri should not be NULL");
     return;
   }
 
@@ -187,10 +189,12 @@ _load_project (GESProject * project, GESTimeline * timeline, GError ** error)
   }
 
   if (priv->formatter_asset == NULL)
-    priv->formatter_asset = _find_formatter_asset_for_uri (priv->uri);
+    priv->formatter_asset = _find_formatter_asset_for_id (priv->uri);
 
-  if (priv->formatter_asset == NULL)
+  if (priv->formatter_asset == NULL) {
+    lerr = g_error_new (GES_ERROR, 0, "Could not find a suitable formatter");
     goto failed;
+  }
 
   formatter = GES_FORMATTER (ges_asset_extract (priv->formatter_asset, &lerr));
   if (lerr) {
@@ -249,6 +253,99 @@ ges_project_extract (GESAsset * project, GError ** error)
   return NULL;
 }
 
+static void
+_add_media_new_paths_recursing (const gchar * value)
+{
+  GFileInfo *info;
+  GFileEnumerator *fenum;
+  GFile *file = g_file_new_for_uri (value);
+
+  if (!(fenum = g_file_enumerate_children (file,
+              "standard::*", G_FILE_QUERY_INFO_NONE, NULL, NULL))) {
+    GST_INFO ("%s is not a folder", value);
+
+    goto done;
+  }
+
+  GST_INFO ("Adding folder: %s", value);
+  g_ptr_array_add (new_paths, g_strdup (value));
+  for (info = g_file_enumerator_next_file (fenum, NULL, NULL);
+      info; info = g_file_enumerator_next_file (fenum, NULL, NULL)) {
+
+    if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY) {
+      GFile *f = g_file_enumerator_get_child (fenum, info);
+      gchar *uri = g_file_get_uri (f);
+
+      _add_media_new_paths_recursing (uri);
+      gst_object_unref (f);
+      g_free (uri);
+    }
+  }
+
+done:
+  gst_object_unref (file);
+  if (fenum)
+    gst_object_unref (fenum);
+}
+
+gboolean
+ges_add_missing_uri_relocation_uri (const gchar * uri, gboolean recurse)
+{
+  g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
+
+  if (new_paths == NULL)
+    new_paths = g_ptr_array_new_with_free_func (g_free);
+
+  if (recurse)
+    _add_media_new_paths_recursing (uri);
+  else
+    g_ptr_array_add (new_paths, g_strdup (uri));
+
+  return TRUE;
+}
+
+static gchar *
+ges_missing_uri_default (GESProject * self, GError * error,
+    GESAsset * wrong_asset)
+{
+  guint i;
+  const gchar *old_uri = ges_asset_get_id (wrong_asset);
+
+  if (new_paths == NULL)
+    return NULL;
+
+  if (tried_uris == NULL)
+    tried_uris = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+  for (i = 0; i < new_paths->len; i++) {
+    gchar *basename, *res;
+
+    basename = g_path_get_basename (old_uri);
+    res = g_build_filename (new_paths->pdata[i], basename, NULL);
+    g_free (basename);
+
+    if (g_strcmp0 (old_uri, res) == 0) {
+      g_hash_table_add (tried_uris, res);
+    } else if (g_hash_table_lookup (tried_uris, res)) {
+      GST_DEBUG_OBJECT (self, "File already tried: %s", res);
+      g_free (res);
+    } else {
+      g_hash_table_add (tried_uris, g_strdup (res));
+      GST_DEBUG_OBJECT (self, "Trying: %s\n", res);
+      return res;
+    }
+  }
+
+  return NULL;
+}
+
+static void
+ges_uri_assets_validate_uri (const gchar * nid)
+{
+  if (tried_uris)
+    g_hash_table_remove (tried_uris, nid);
+}
+
 /* GObject vmethod implementation */
 static void
 _dispose (GObject * object)
@@ -318,7 +415,7 @@ ges_project_class_init (GESProjectClass * klass)
   g_type_class_add_private (klass, sizeof (GESProjectPrivate));
 
   klass->asset_added = NULL;
-  klass->missing_uri = NULL;
+  klass->missing_uri = ges_missing_uri_default;
   klass->loading_error = NULL;
   klass->asset_removed = NULL;
   object_class->get_property = (GObjectGetPropertyFunc) _get_property;
@@ -540,7 +637,12 @@ gboolean
 ges_project_set_loaded (GESProject * project, GESFormatter * formatter)
 {
   GST_INFO_OBJECT (project, "Emit project loaded");
-  ges_timeline_commit (formatter->timeline);
+  if (GST_STATE (formatter->timeline) < GST_STATE_PAUSED) {
+    timeline_fill_gaps (formatter->timeline);
+  } else {
+    ges_timeline_commit (formatter->timeline);
+  }
+
   g_signal_emit (project, _signals[LOADED_SIGNAL], 0, formatter->timeline);
 
   /* We are now done with that formatter */
@@ -602,6 +704,79 @@ ges_project_create_asset (GESProject * project, const gchar * id,
   return TRUE;
 }
 
+/**
+ * ges_project_create_asset_sync:
+ * @project: A #GESProject
+ * @id: (allow-none): The id of the asset to create and add to @project
+ * @extractable_type: The #GType of the asset to create
+ * @error: A #GError to be set in case of error
+ *
+ * Create and add a #GESAsset to @project. You should connect to the
+ * "asset-added" signal to get the asset when it finally gets added to
+ * @project
+ *
+ * Returns: The newly created #GESAsset
+ */
+GESAsset *
+ges_project_create_asset_sync (GESProject * project, const gchar * id,
+    GType extractable_type, GError ** error)
+{
+  GESAsset *asset;
+  gchar *possible_id = NULL;
+  gboolean retry = TRUE;
+
+  g_return_val_if_fail (GES_IS_PROJECT (project), FALSE);
+  g_return_val_if_fail (g_type_is_a (extractable_type, GES_TYPE_EXTRACTABLE),
+      FALSE);
+
+  if (id == NULL)
+    id = g_type_name (extractable_type);
+
+  if ((asset = g_hash_table_lookup (project->priv->assets, id)))
+    return asset;
+  else if (g_hash_table_lookup (project->priv->loading_assets, id) ||
+      g_hash_table_lookup (project->priv->loaded_with_error, id))
+    return NULL;
+
+  /* TODO Add a GCancellable somewhere in our API */
+  while (retry) {
+
+    if (g_type_is_a (extractable_type, GES_TYPE_URI_CLIP)) {
+      asset = GES_ASSET (ges_uri_clip_asset_request_sync (id, error));
+    } else {
+      asset = ges_asset_request (extractable_type, id, error);
+    }
+
+    if (asset) {
+      retry = FALSE;
+
+      if (possible_id) {
+        g_free (possible_id);
+        ges_uri_assets_validate_uri (id);
+      }
+
+      break;
+    } else {
+      GESAsset *tmpasset;
+
+      tmpasset = ges_asset_cache_lookup (extractable_type, id);
+      possible_id = ges_project_try_updating_id (project, tmpasset, *error);
+
+      if (possible_id == NULL)
+        return NULL;
+
+      if (error)
+        g_clear_error (error);
+
+      id = possible_id;
+    }
+  }
+
+  ges_project_add_asset (project, asset);
+
+  return asset;
+}
+
 /**
  * ges_project_add_asset:
  * @project: A #GESProject
@@ -619,7 +794,7 @@ ges_project_add_asset (GESProject * project, GESAsset * asset)
   g_return_val_if_fail (GES_IS_PROJECT (project), FALSE);
 
   if (g_hash_table_lookup (project->priv->assets, ges_asset_get_id (asset)))
-    return FALSE;
+    return TRUE;
 
   g_hash_table_insert (project->priv->assets,
       g_strdup (ges_asset_get_id (asset)), gst_object_ref (asset));
@@ -819,6 +994,9 @@ ges_project_new (const gchar * uri)
   if (project && uri)
     ges_project_set_uri (project, uri);
 
+  if (uri == NULL)
+    g_free (id);
+
   return project;
 }
 
@@ -838,8 +1016,7 @@ ges_project_load (GESProject * project, GESTimeline * timeline, GError ** error)
   g_return_val_if_fail (GES_IS_TIMELINE (timeline), FALSE);
   g_return_val_if_fail (GES_IS_PROJECT (project), FALSE);
   g_return_val_if_fail (ges_project_get_uri (project), FALSE);
-  g_return_val_if_fail (
-      (ges_extractable_get_asset (GES_EXTRACTABLE (timeline)) == NULL), FALSE);
+  g_return_val_if_fail (timeline->tracks == NULL, FALSE);
 
   if (!_load_project (project, timeline, error))
     return FALSE;
