@@ -83,7 +83,10 @@ _request_new_pad (GstElement * element, GstPadTemplate * templ,
   PadInfos *infos = g_slice_new0 (PadInfos);
   GESSmartAdder *self = GES_SMART_ADDER (element);
 
-  infos->adder_pad = gst_element_request_pad (self->adder, templ, NULL, caps);
+  infos->adder_pad = gst_element_request_pad (self->adder,
+      gst_element_class_get_pad_template (GST_ELEMENT_GET_CLASS (self->adder),
+          "sink_%u"), NULL, caps);
+
   if (infos->adder_pad == NULL) {
     GST_WARNING_OBJECT (element, "Could not get any pad from GstAdder");
     g_slice_free (PadInfos, infos);
@@ -147,6 +150,19 @@ _release_pad (GstElement * element, GstPad * pad)
 /****************************************************
  *              GObject vmethods                    *
  ****************************************************/
+static void
+ges_smart_adder_dispose (GObject * object)
+{
+  GESSmartAdder *self = GES_SMART_ADDER (object);
+
+  if (self->pads_infos) {
+    g_hash_table_unref (self->pads_infos);
+    self->pads_infos = NULL;
+  }
+
+  G_OBJECT_CLASS (ges_smart_adder_parent_class)->dispose (object);
+}
+
 static void
 ges_smart_adder_finalize (GObject * object)
 {
@@ -178,6 +194,7 @@ ges_smart_adder_class_init (GESSmartAdderClass * klass)
   element_class->request_new_pad = GST_DEBUG_FUNCPTR (_request_new_pad);
   element_class->release_pad = GST_DEBUG_FUNCPTR (_release_pad);
 
+  object_class->dispose = ges_smart_adder_dispose;
   object_class->finalize = ges_smart_adder_finalize;
 }
 
@@ -188,7 +205,7 @@ ges_smart_adder_init (GESSmartAdder * self)
 
   g_mutex_init (&self->lock);
 
-  self->adder = gst_element_factory_make ("adder", "smart-adder-adder");
+  self->adder = gst_element_factory_make ("audiomixer", "smart-adder-adder");
   gst_bin_add (GST_BIN (self), self->adder);
 
   pad = gst_element_get_static_pad (self->adder, "src");
@@ -205,10 +222,17 @@ ges_smart_adder_init (GESSmartAdder * self)
 GstElement *
 ges_smart_adder_new (GESTrack * track)
 {
-  GESSmartAdder *self = g_object_new (GES_TYPE_SMART_ADDER, NULL);
+  GESSmartAdder *self;
+  GstCaps *caps;
+
+  self = g_object_new (GES_TYPE_SMART_ADDER, NULL);
+
   self->track = track;
 
   /* FIXME Make adder smart and let it properly negotiate caps! */
-  g_object_set (self->adder, "caps", gst_caps_from_string (DEFAULT_CAPS), NULL);
+  caps = gst_caps_from_string (DEFAULT_CAPS);
+  g_object_set (self->adder, "caps", caps, NULL);
+  gst_caps_unref (caps);
+
   return GST_ELEMENT (self);
 }
