@@ -22,6 +22,70 @@
  * SECTION:gestitlesource
  * @short_description: render stand-alone text titles
  * 
+ * #GESTitleSource is a GESTimelineElement that implements the notion
+ * of titles in GES.
+ *
+ * <refsect1 id="GESTileSource.children_properties" role="properties">
+ * <title role="children_properties.title">Children Properties</title>
+ * <para>You can use the following children properties through the
+ * #ges_track_element_set_child_property and alike set of methods:</para>
+ * <informaltable frame="none">
+ * <tgroup cols="3">
+ * <colspec colname="properties_type" colwidth="150px"/>
+ * <colspec colname="properties_name" colwidth="200px"/>
+ * <colspec colname="properties_flags" colwidth="400px"/>
+ * <tbody>
+ * <row>
+ *  <entry role="property_type"><link linkend="guint"><type>guint</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--background">background</link></entry>
+ *  <entry>The color of the background</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="guint"><type>guint</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--color">color</link></entry>
+ *  <entry>The color of the text</entry>
+ * </row>
+ * <row>
+ *   <entry role="property_type"><link linkend="gchar"><type>gchar</type></link></entry>
+ *   <entry role="property_name"><link linkend="GESTileSource--font-desc">font-desc</link></entry>
+ *   <entry>Pango font description string</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="GESTextHAlign"><type>GESTextHAlign</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--halignment">halignment</link></entry>
+ *  <entry>Horizontal alignment of the text</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="gchar"><type>gchar</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--text">text</link></entry>
+ *  <entry>The text to be rendered</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="GESTextVAlign"><type>GESTextVAlign</type></link>
+ *  </entry><entry role="property_name"><link linkend="GESTileSource--valignment">valignment</link>
+ *  </entry><entry>Vertical alignent of the text</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="gdouble"><type>gdouble</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--xpos">xpos</link></entry>
+ *  <entry>The horizontal position of the text</entry>
+ * </row>
+ * <row><entry role="property_type"><link linkend="gdouble"><type>gdouble</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--ypos">ypos</link></entry>
+ *  <entry>The vertical position of the text</entry>
+ * </row>
+ * <row><entry role="property_type"><link linkend="gboolean"><type>gboolean</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--shaded-background">shaded-background</link></entry>
+ *  <entry>Whether to shade the background under the text area</entry>
+ * </row>
+ * <row><entry role="property_type"><link linkend="guint"><type>guint</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESTileSource--outline-color">outline-color</link></entry>
+ *  <entry>Color to use for outline the text (big-endian ARGB).</entry>
+ * </row>
+ * </tbody>
+ * </tgroup>
+ * </informaltable>
+ * </refsect1>
  */
 
 #include "ges-internal.h"
@@ -60,17 +124,43 @@ static void ges_title_source_set_property (GObject * object, guint
 
 static GstElement *ges_title_source_create_source (GESTrackElement * self);
 
+static gboolean
+_lookup_child (GESTrackElement * object,
+    const gchar * prop_name, GstElement ** element, GParamSpec ** pspec)
+{
+  gboolean res;
+
+  gchar *clean_name;
+
+  if (!g_strcmp0 (prop_name, "background"))
+    clean_name = g_strdup ("foreground-color");
+  else if (!g_strcmp0 (prop_name, "GstTextOverlay:background"))
+    clean_name = g_strdup ("foreground-color");
+  else
+    clean_name = g_strdup (prop_name);
+
+  res =
+      GES_TRACK_ELEMENT_CLASS (ges_title_source_parent_class)->lookup_child
+      (object, clean_name, element, pspec);
+
+  g_free (clean_name);
+
+  return res;
+}
+
 static void
 ges_title_source_class_init (GESTitleSourceClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GESVideoSourceClass *source_class = GES_VIDEO_SOURCE_CLASS (klass);
+  GESTrackElementClass *track_element_class = GES_TRACK_ELEMENT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GESTitleSourcePrivate));
 
   object_class->get_property = ges_title_source_get_property;
   object_class->set_property = ges_title_source_set_property;
   object_class->dispose = ges_title_source_dispose;
+  track_element_class->lookup_child = _lookup_child;
 
   source_class->create_source = ges_title_source_create_source;
 }
@@ -141,11 +231,16 @@ ges_title_source_set_property (GObject * object,
 static GstElement *
 ges_title_source_create_source (GESTrackElement * object)
 {
-  GESTitleSource *self = GES_TITLE_SOURCE (object);
-  GESTitleSourcePrivate *priv = self->priv;
   GstElement *topbin, *background, *text;
   GstPad *src, *pad;
 
+  GESTitleSource *self = GES_TITLE_SOURCE (object);
+  GESTitleSourcePrivate *priv = self->priv;
+  const gchar *bg_props[] = { "pattern", "foreground-color", NULL };
+  const gchar *text_props[] = { "text", "font-desc", "valignment", "halignment",
+    "color", "xpos", "ypos", "outline-color", "shaded-background", NULL
+  };
+
   topbin = gst_bin_new ("titlesrc-bin");
   background = gst_element_factory_make ("videotestsrc", "titlesrc-bg");
 
@@ -158,14 +253,15 @@ ges_title_source_create_source (GESTrackElement * object)
   }
   g_object_set (text, "valignment", (gint) priv->valign, "halignment",
       (gint) priv->halign, NULL);
+  g_object_set (text, "color", (guint) self->priv->color, NULL);
+  g_object_set (text, "xpos", (gdouble) self->priv->xpos, NULL);
+  g_object_set (text, "ypos", (gdouble) self->priv->ypos, NULL);
+
 
   g_object_set (background, "pattern", (gint) GES_VIDEO_TEST_PATTERN_SOLID,
       NULL);
   g_object_set (background, "foreground-color", (guint) self->priv->background,
       NULL);
-  g_object_set (text, "color", (guint) self->priv->color, NULL);
-  g_object_set (text, "xpos", (gdouble) self->priv->xpos, NULL);
-  g_object_set (text, "ypos", (gdouble) self->priv->ypos, NULL);
 
   gst_bin_add_many (GST_BIN (topbin), background, text, NULL);
 
@@ -183,6 +279,10 @@ ges_title_source_create_source (GESTrackElement * object)
   priv->text_el = text;
   priv->background_el = background;
 
+  ges_track_element_add_children_props (object, text, NULL, NULL, text_props);
+  ges_track_element_add_children_props (object, background, NULL, NULL,
+      bg_props);
+
   return topbin;
 }
 
@@ -193,6 +293,9 @@ ges_title_source_create_source (GESTrackElement * object)
  * made.
  * 
  * Sets the text this track element will render.
+ *
+ * Deprecated: use ges_track_element_get/set_children_properties on the
+ * GESTrackElement instead
  */
 
 void
@@ -343,7 +446,12 @@ ges_title_source_set_ypos (GESTitleSource * self, gdouble position)
 const gchar *
 ges_title_source_get_text (GESTitleSource * source)
 {
-  return source->priv->text;
+  gchar *text;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source), "text",
+      &text, NULL);
+
+  return text;
 }
 
 /**
@@ -358,7 +466,12 @@ ges_title_source_get_text (GESTitleSource * source)
 const gchar *
 ges_title_source_get_font_desc (GESTitleSource * source)
 {
-  return source->priv->font_desc;
+  gchar *font_desc;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source),
+      "font-desc", &font_desc, NULL);
+
+  return font_desc;
 }
 
 /**
@@ -372,7 +485,12 @@ ges_title_source_get_font_desc (GESTitleSource * source)
 GESTextHAlign
 ges_title_source_get_halignment (GESTitleSource * source)
 {
-  return source->priv->halign;
+  GESTextHAlign halign;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source),
+      "halignment", &halign, NULL);
+
+  return halign;
 }
 
 /**
@@ -386,7 +504,12 @@ ges_title_source_get_halignment (GESTitleSource * source)
 GESTextVAlign
 ges_title_source_get_valignment (GESTitleSource * source)
 {
-  return source->priv->valign;
+  GESTextVAlign valign;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source),
+      "valignment", &valign, NULL);
+
+  return valign;
 }
 
 /**
@@ -400,7 +523,12 @@ ges_title_source_get_valignment (GESTitleSource * source)
 const guint32
 ges_title_source_get_text_color (GESTitleSource * source)
 {
-  return source->priv->color;
+  guint32 color;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source), "color",
+      &color, NULL);
+
+  return color;
 }
 
 /**
@@ -414,7 +542,12 @@ ges_title_source_get_text_color (GESTitleSource * source)
 const guint32
 ges_title_source_get_background_color (GESTitleSource * source)
 {
-  return source->priv->background;
+  guint32 color;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source),
+      "foreground-color", &color, NULL);
+
+  return color;
 }
 
 /**
@@ -428,7 +561,12 @@ ges_title_source_get_background_color (GESTitleSource * source)
 const gdouble
 ges_title_source_get_xpos (GESTitleSource * source)
 {
-  return source->priv->xpos;
+  gdouble xpos;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source), "xpos",
+      &xpos, NULL);
+
+  return xpos;
 }
 
 /**
@@ -442,7 +580,12 @@ ges_title_source_get_xpos (GESTitleSource * source)
 const gdouble
 ges_title_source_get_ypos (GESTitleSource * source)
 {
-  return source->priv->ypos;
+  gdouble ypos;
+
+  ges_track_element_get_child_properties (GES_TRACK_ELEMENT (source), "ypos",
+      &ypos, NULL);
+
+  return ypos;
 }
 
 /**
