@@ -32,8 +32,8 @@
 G_DEFINE_TYPE (GESXmlFormatter, ges_xml_formatter, GES_TYPE_BASE_XML_FORMATTER);
 
 #define API_VERSION 0
-#define MINOR_VERSION 1
-#define VERSION 0.1
+#define MINOR_VERSION 2
+#define VERSION 0.2
 
 #define COLLECT_STR_OPT (G_MARKUP_COLLECT_STRING | G_MARKUP_COLLECT_OPTIONAL)
 
@@ -45,6 +45,12 @@ struct _GESXmlFormatterPrivate
   gboolean project_opened;
 
   GString *str;
+
+  GHashTable *element_id;
+
+  guint nbelements;
+
+  guint min_version;
 };
 
 static inline void
@@ -52,8 +58,8 @@ _parse_ges_element (GMarkupParseContext * context, const gchar * element_name,
     const gchar ** attribute_names, const gchar ** attribute_values,
     GESXmlFormatter * self, GError ** error)
 {
+  guint api_version;
   const gchar *version, *properties;
-  guint api_version, min_version;
 
   gchar **split_version = NULL;
 
@@ -80,8 +86,8 @@ _parse_ges_element (GMarkupParseContext * context, const gchar * element_name,
   if (errno || api_version != API_VERSION)
     goto stroull_failed;
 
-  min_version = g_ascii_strtoull (split_version[1], NULL, 10);
-  if (min_version > MINOR_VERSION)
+  self->priv->min_version = g_ascii_strtoull (split_version[1], NULL, 10);
+  if (self->priv->min_version > MINOR_VERSION)
     goto failed;
 
   _GET_PRIV (self)->ges_opened = TRUE;
@@ -153,7 +159,7 @@ _parse_encoding_profile (GMarkupParseContext * context,
 
   ges_base_xml_formatter_add_encoding_profile (GES_BASE_XML_FORMATTER (self),
       type, NULL, name, description, capsformat, preset, preset_name, 0, 0,
-      NULL, 0, FALSE, NULL, error);
+      NULL, 0, FALSE, NULL, TRUE, error);
 }
 
 static inline void
@@ -161,12 +167,13 @@ _parse_stream_profile (GMarkupParseContext * context,
     const gchar * element_name, const gchar ** attribute_names,
     const gchar ** attribute_values, GESXmlFormatter * self, GError ** error)
 {
-  gboolean variableframerate = FALSE;
+  gboolean variableframerate = FALSE, enabled = TRUE;
   guint id = 0, presence = 0, pass = 0;
   GstCaps *format_caps = NULL, *restriction_caps = NULL;
   const gchar *parent, *strid, *type, *strpresence, *format = NULL,
       *name = NULL, *description = NULL, *preset, *preset_name =
-      NULL, *restriction = NULL, *strpass = NULL, *strvariableframerate = NULL;
+      NULL, *restriction = NULL, *strpass = NULL, *strvariableframerate = NULL,
+      *strenabled = NULL;
 
   /* FIXME Looks like there is a bug in that function, if we put the parent
    * at the beginning it set %NULL and not the real value... :/ */
@@ -183,6 +190,7 @@ _parse_stream_profile (GMarkupParseContext * context,
           COLLECT_STR_OPT, "restriction", &restriction,
           COLLECT_STR_OPT, "pass", &strpass,
           COLLECT_STR_OPT, "variableframerate", &strvariableframerate,
+          COLLECT_STR_OPT, "enabled", &strenabled,
           G_MARKUP_COLLECT_STRING, "parent", &parent, G_MARKUP_COLLECT_INVALID))
     return;
 
@@ -209,6 +217,12 @@ _parse_stream_profile (GMarkupParseContext * context,
       goto convertion_failed;
   }
 
+  if (strenabled) {
+    enabled = g_ascii_strtoll (strenabled, NULL, 10);
+    if (errno)
+      goto convertion_failed;
+  }
+
   if (format)
     format_caps = gst_caps_from_string (format);
 
@@ -217,7 +231,8 @@ _parse_stream_profile (GMarkupParseContext * context,
 
   ges_base_xml_formatter_add_encoding_profile (GES_BASE_XML_FORMATTER (self),
       type, parent, name, description, format_caps, preset, preset_name, id,
-      presence, restriction_caps, pass, variableframerate, NULL, error);
+      presence, restriction_caps, pass, variableframerate, NULL, enabled,
+      error);
 
   return;
 
@@ -559,6 +574,43 @@ _parse_binding (GMarkupParseContext * context, const gchar * element_name,
       property_name, (gint) g_ascii_strtoll (mode, NULL, 10), track_id, list);
 }
 
+static inline void
+_parse_source (GMarkupParseContext * context, const gchar * element_name,
+    const gchar ** attribute_names, const gchar ** attribute_values,
+    GESXmlFormatter * self, GError ** error)
+{
+  GstStructure *children_props = NULL;
+  const gchar *track_id = NULL, *children_properties = NULL;
+
+  if (!g_markup_collect_attributes (element_name, attribute_names,
+          attribute_values, error,
+          G_MARKUP_COLLECT_STRING, "track-id", &track_id,
+          COLLECT_STR_OPT, "children-properties", &children_properties,
+          G_MARKUP_COLLECT_INVALID)) {
+    return;
+  }
+
+  if (children_properties) {
+    children_props = gst_structure_from_string (children_properties, NULL);
+    if (children_props == NULL)
+      goto wrong_children_properties;
+  }
+
+  ges_base_xml_formatter_add_source (GES_BASE_XML_FORMATTER (self), track_id,
+      children_props);
+
+  if (children_props)
+    gst_structure_free (children_props);
+
+  return;
+
+wrong_children_properties:
+  g_set_error (error, G_MARKUP_ERROR,
+      G_MARKUP_ERROR_INVALID_CONTENT,
+      "element '%s', children properties '%s', could no be deserialized",
+      element_name, children_properties);
+}
+
 static inline void
 _parse_effect (GMarkupParseContext * context, const gchar * element_name,
     const gchar ** attribute_names, const gchar ** attribute_values,
@@ -634,6 +686,44 @@ wrong_type:
       "element '%s', %s not a GESBaseEffect'", element_name, strtype);
 }
 
+
+static inline void
+_parse_group (GMarkupParseContext * context, const gchar * element_name,
+    const gchar ** attribute_names, const gchar ** attribute_values,
+    GESXmlFormatter * self, GError ** error)
+{
+  const gchar *id, *properties;
+
+  if (!g_markup_collect_attributes (element_name, attribute_names,
+          attribute_values, error,
+          G_MARKUP_COLLECT_STRING, "id", &id,
+          G_MARKUP_COLLECT_STRING, "properties", &properties,
+          G_MARKUP_COLLECT_INVALID)) {
+    return;
+  }
+
+  ges_base_xml_formatter_add_group (GES_BASE_XML_FORMATTER (self), id,
+      properties);
+}
+
+static inline void
+_parse_group_child (GMarkupParseContext * context, const gchar * element_name,
+    const gchar ** attribute_names, const gchar ** attribute_values,
+    GESXmlFormatter * self, GError ** error)
+{
+  const gchar *child_id, *name;
+
+  if (!g_markup_collect_attributes (element_name, attribute_names,
+          attribute_values, error,
+          G_MARKUP_COLLECT_STRING, "id", &child_id,
+          G_MARKUP_COLLECT_STRING, "name", &name, G_MARKUP_COLLECT_INVALID)) {
+    return;
+  }
+
+  ges_base_xml_formatter_last_group_add_child (GES_BASE_XML_FORMATTER (self),
+      child_id, name);
+}
+
 static void
 _parse_element_start (GMarkupParseContext * context, const gchar * element_name,
     const gchar ** attribute_names, const gchar ** attribute_values,
@@ -668,12 +758,21 @@ _parse_element_start (GMarkupParseContext * context, const gchar * element_name,
   else if (g_strcmp0 (element_name, "clip") == 0)
     _parse_clip (context, element_name, attribute_names,
         attribute_values, self, error);
+  else if (g_strcmp0 (element_name, "source") == 0)
+    _parse_source (context, element_name, attribute_names,
+        attribute_values, self, error);
   else if (g_strcmp0 (element_name, "effect") == 0)
     _parse_effect (context, element_name, attribute_names,
         attribute_values, self, error);
   else if (g_strcmp0 (element_name, "binding") == 0)
     _parse_binding (context, element_name, attribute_names,
         attribute_values, self, error);
+  else if (g_strcmp0 (element_name, "group") == 0)
+    _parse_group (context, element_name, attribute_names,
+        attribute_values, self, error);
+  else if (g_strcmp0 (element_name, "child") == 0)
+    _parse_group_child (context, element_name, attribute_names,
+        attribute_values, self, error);
   else
     GST_LOG_OBJECT (self, "Element %s not handled", element_name);
 }
@@ -683,6 +782,15 @@ _parse_element_end (GMarkupParseContext * context,
     const gchar * element_name, gpointer self, GError ** error)
 {
   /*GESXmlFormatterPrivate *priv = _GET_PRIV (self); */
+  if (g_strcmp0 (element_name, "ges") == 0 && GES_FORMATTER (self)->project) {
+    gchar *version = g_strdup_printf ("%d.%d",
+        API_VERSION, GES_XML_FORMATTER (self)->priv->min_version);
+
+    ges_meta_container_set_string (GES_META_CONTAINER (GES_FORMATTER
+            (self)->project), GES_META_FORMAT_VERSION, version);
+
+    g_free (version);
+  }
 }
 
 static void
@@ -709,14 +817,39 @@ append_escaped (GString * str, gchar * tmpstr)
 static inline gboolean
 _can_serialize_spec (GParamSpec * spec)
 {
-  if (spec->flags & G_PARAM_WRITABLE && !(spec->flags & G_PARAM_CONSTRUCT_ONLY)
-      && !g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (spec), G_TYPE_OBJECT)
-      && (!(g_type_is_a (spec->owner_type, GST_TYPE_OBJECT) &&
-              !g_strcmp0 (spec->name, "name")))
-      && G_PARAM_SPEC_VALUE_TYPE (spec) != G_TYPE_GTYPE)
-    return TRUE;
+  if (!(spec->flags & G_PARAM_WRITABLE)) {
+    GST_DEBUG ("%s from %s is not writable",
+        spec->name, g_type_name (spec->owner_type));
 
-  return FALSE;
+    return FALSE;
+  } else if (spec->flags & G_PARAM_CONSTRUCT_ONLY) {
+    GST_DEBUG ("%s from %s is construct only",
+        spec->name, g_type_name (spec->owner_type));
+
+    return FALSE;
+  } else if (spec->flags & GES_PARAM_NO_SERIALIZATION &&
+      g_type_is_a (spec->owner_type, GES_TYPE_TIMELINE_ELEMENT)) {
+    GST_DEBUG ("%s from %s is set as GES_PARAM_NO_SERIALIZATION",
+        spec->name, g_type_name (spec->owner_type));
+
+    return FALSE;
+  } else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (spec), G_TYPE_OBJECT)) {
+    GST_DEBUG ("%s from %s contains GObject, can't serialize that.",
+        spec->name, g_type_name (spec->owner_type));
+
+    return FALSE;
+  } else if ((g_type_is_a (spec->owner_type, GST_TYPE_OBJECT) &&
+          !g_strcmp0 (spec->name, "name"))) {
+
+    GST_DEBUG ("We do not want to serialize the name of GstObjects.");
+    return FALSE;
+  } else if (G_PARAM_SPEC_VALUE_TYPE (spec) == G_TYPE_GTYPE) {
+    GST_DEBUG ("%s from %s contains a GType, can't serialize.",
+        spec->name, g_type_name (spec->owner_type));
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 static inline void
@@ -797,7 +930,7 @@ _save_assets (GString * str, GESProject * project)
 }
 
 static inline void
-_save_tracks (GString * str, GESTimeline * timeline)
+_save_tracks (GESXmlFormatter * self, GString * str, GESTimeline * timeline)
 {
   gchar *strtmp, *metas;
   GESTrack *track;
@@ -823,6 +956,37 @@ _save_tracks (GString * str, GESTimeline * timeline)
   g_list_free_full (tracks, gst_object_unref);
 }
 
+static inline void
+_save_children_properties (GString * str, GESTrackElement * trackelement)
+{
+  GstStructure *structure;
+  GParamSpec **pspecs, *spec;
+  guint i, n_props;
+
+  pspecs = ges_track_element_list_children_properties (trackelement, &n_props);
+
+  structure = gst_structure_new_empty ("properties");
+  for (i = 0; i < n_props; i++) {
+    GValue val = { 0 };
+    spec = pspecs[i];
+
+    if (_can_serialize_spec (spec)) {
+      _init_value_from_spec_for_serialization (&val, spec);
+      ges_track_element_get_child_property_by_pspec (trackelement, spec, &val);
+      gst_structure_set_value (structure, spec->name, &val);
+      g_value_unset (&val);
+    }
+    g_param_spec_unref (spec);
+  }
+  g_free (pspecs);
+
+  append_escaped (str,
+      g_markup_printf_escaped (" children-properties='%s'",
+          gst_structure_to_string (structure)));
+
+  gst_structure_free (structure);
+}
+
 /* TODO : Use this function for every track element with controllable properties */
 static inline void
 _save_keyframes (GString * str, GESTrackElement * trackelement, gint index)
@@ -831,7 +995,8 @@ _save_keyframes (GString * str, GESTrackElement * trackelement, gint index)
   GHashTableIter iter;
   gpointer key, value;
 
-  bindings_hashtable = ges_track_element_get_bindings_hashtable (trackelement);
+  bindings_hashtable =
+      ges_track_element_get_all_control_bindings (trackelement);
 
   g_hash_table_iter_init (&iter, bindings_hashtable);
 
@@ -839,11 +1004,13 @@ _save_keyframes (GString * str, GESTrackElement * trackelement, gint index)
   while (g_hash_table_iter_next (&iter, &key, &value)) {
     if (GST_IS_DIRECT_CONTROL_BINDING ((GstControlBinding *) value)) {
       GstControlSource *source;
+      gboolean absolute = FALSE;
       GstDirectControlBinding *binding;
 
       binding = (GstDirectControlBinding *) value;
 
-      g_object_get (binding, "control-source", &source, NULL);
+      g_object_get (binding, "control-source", &source,
+          "absolute", &absolute, NULL);
 
       if (GST_IS_INTERPOLATION_CONTROL_SOURCE (source)) {
         GList *timed_values, *tmp;
@@ -851,8 +1018,9 @@ _save_keyframes (GString * str, GESTrackElement * trackelement, gint index)
 
         append_escaped (str,
             g_markup_printf_escaped
-            ("            <binding type='direct' source_type='interpolation' property='%s'",
-                (gchar *) key));
+            ("            <binding type='%s' source_type='interpolation' property='%s'",
+                absolute ? "direct-absolute" : "direct", (gchar *) key));
+
         g_object_get (source, "mode", &mode, NULL);
         append_escaped (str, g_markup_printf_escaped (" mode='%d'", mode));
         append_escaped (str, g_markup_printf_escaped (" track_id='%d'", index));
@@ -873,7 +1041,7 @@ _save_keyframes (GString * str, GESTrackElement * trackelement, gint index)
       } else
         GST_DEBUG ("control source not in [interpolation]");
     } else
-      GST_DEBUG ("Binding type not in [direct]");
+      GST_DEBUG ("Binding type not in [direct, direct-absolute]");
   }
 }
 
@@ -883,10 +1051,17 @@ _save_effect (GString * str, guint clip_id, GESTrackElement * trackelement,
 {
   GESTrack *tck;
   GList *tmp, *tracks;
-  GstStructure *structure;
   gchar *properties, *metas;
-  GParamSpec **pspecs, *spec;
-  guint j, n_props = 0, track_id = 0;
+  guint track_id = 0;
+  gboolean serialize;
+
+  g_object_get (trackelement, "serialize", &serialize, NULL);
+  if (!serialize) {
+
+    GST_DEBUG_OBJECT (trackelement, "Should not be serialized");
+
+    return;
+  }
 
   tck = ges_track_element_get_track (trackelement);
   if (tck == NULL) {
@@ -916,41 +1091,22 @@ _save_effect (GString * str, guint clip_id, GESTrackElement * trackelement,
   g_free (properties);
   g_free (metas);
 
-  pspecs = ges_track_element_list_children_properties (trackelement, &n_props);
-  structure = gst_structure_new_empty ("properties");
-  for (j = 0; j < n_props; j++) {
-    GValue val = { 0 };
-
-    spec = pspecs[j];
-    if (_can_serialize_spec (spec)) {
-      _init_value_from_spec_for_serialization (&val, spec);
-      ges_track_element_get_child_property_by_pspec (trackelement, spec, &val);
-      gst_structure_set_value (structure, spec->name, &val);
-      g_value_unset (&val);
-    }
-    g_param_spec_unref (spec);
-  }
-  g_free (pspecs);
-
-  append_escaped (str,
-      g_markup_printf_escaped (" children-properties='%s'>\n",
-          gst_structure_to_string (structure)));
+  _save_children_properties (str, trackelement);
+  append_escaped (str, g_markup_printf_escaped (">\n"));
 
   _save_keyframes (str, trackelement, -1);
 
   append_escaped (str, g_markup_printf_escaped ("          </effect>\n"));
-  gst_structure_free (structure);
 }
 
 static inline void
-_save_layers (GString * str, GESTimeline * timeline)
+_save_layers (GESXmlFormatter * self, GString * str, GESTimeline * timeline)
 {
   gchar *properties, *metas;
   GESLayer *layer;
   GESClip *clip;
   GList *tmplayer, *tmpclip, *clips;
-
-  guint nbclips = 0;
+  GESXmlFormatterPrivate *priv = self->priv;
 
   for (tmplayer = timeline->layers; tmplayer; tmplayer = tmplayer->next) {
     guint priority;
@@ -971,8 +1127,16 @@ _save_layers (GString * str, GESTimeline * timeline)
       GList *effects, *tmpeffect;
       GList *tmptrackelement;
       GList *tracks;
+      gboolean serialize;
 
       clip = GES_CLIP (tmpclip->data);
+
+      g_object_get (clip, "serialize", &serialize, NULL);
+      if (!serialize) {
+        GST_DEBUG_OBJECT (clip, "Should not be serialized");
+        continue;
+      }
+
       effects = ges_clip_get_top_effects (clip);
 
       /* We escape all mandatrorry properties that are handled sparetely
@@ -984,45 +1148,120 @@ _save_layers (GString * str, GESTimeline * timeline)
           g_markup_printf_escaped ("        <clip id='%i' asset-id='%s'"
               " type-name='%s' layer-priority='%i' track-types='%i' start='%"
               G_GUINT64_FORMAT "' duration='%" G_GUINT64_FORMAT "' inpoint='%"
-              G_GUINT64_FORMAT "' rate='%d' properties='%s' >\n", nbclips,
-              ges_extractable_get_id (GES_EXTRACTABLE (clip)),
+              G_GUINT64_FORMAT "' rate='%d' properties='%s' >\n",
+              priv->nbelements, ges_extractable_get_id (GES_EXTRACTABLE (clip)),
               g_type_name (G_OBJECT_TYPE (clip)), priority,
               ges_clip_get_supported_formats (clip), _START (clip),
               _DURATION (clip), _INPOINT (clip), 0, properties));
       g_free (properties);
 
+      g_hash_table_insert (self->priv->element_id, clip,
+          GINT_TO_POINTER (priv->nbelements));
+
       for (tmpeffect = effects; tmpeffect; tmpeffect = tmpeffect->next)
-        _save_effect (str, nbclips, GES_TRACK_ELEMENT (tmpeffect->data),
-            timeline);
+        _save_effect (str, priv->nbelements,
+            GES_TRACK_ELEMENT (tmpeffect->data), timeline);
 
       tracks = ges_timeline_get_tracks (timeline);
 
       for (tmptrackelement = GES_CONTAINER_CHILDREN (clip); tmptrackelement;
           tmptrackelement = tmptrackelement->next) {
         gint index;
+        gboolean serialize;
 
         if (!GES_IS_SOURCE (tmptrackelement->data))
           continue;
 
+        g_object_get (tmptrackelement->data, "serialize", &serialize, NULL);
+        if (!serialize) {
+          GST_DEBUG_OBJECT (tmptrackelement->data, "Should not be serialized");
+          continue;
+        }
+
         index =
             g_list_index (tracks,
             ges_track_element_get_track (tmptrackelement->data));
+        append_escaped (str,
+            g_markup_printf_escaped ("          <source track-id='%i'", index));
+        _save_children_properties (str, tmptrackelement->data);
+        append_escaped (str, g_markup_printf_escaped (">\n"));
         _save_keyframes (str, tmptrackelement->data, index);
+        append_escaped (str, g_markup_printf_escaped ("          </source>\n"));
       }
 
       g_list_free_full (tracks, gst_object_unref);
 
       g_string_append (str, "        </clip>\n");
 
-      nbclips++;
+      priv->nbelements++;
     }
     g_string_append (str, "      </layer>\n");
   }
 }
 
+static void
+_save_group (GESXmlFormatter * self, GString * str, GList ** seen_groups,
+    GESGroup * group)
+{
+  GList *tmp;
+  gboolean serialize;
+  gchar *properties;
+
+  g_object_get (group, "serialize", &serialize, NULL);
+  if (!serialize) {
+
+    GST_DEBUG_OBJECT (group, "Should not be serialized");
+
+    return;
+  }
+
+  if (g_list_find (*seen_groups, group)) {
+    GST_DEBUG_OBJECT (group, "Already serialized");
+
+    return;
+  }
+
+  *seen_groups = g_list_prepend (*seen_groups, group);
+  for (tmp = GES_CONTAINER_CHILDREN (group); tmp; tmp = tmp->next) {
+    if (GES_IS_GROUP (tmp->data)) {
+      _save_group (self, str, seen_groups,
+          GES_GROUP (GES_TIMELINE_ELEMENT (tmp->data)));
+    }
+  }
+
+  properties = _serialize_properties (G_OBJECT (group), NULL);
+  g_string_append_printf (str, "        <group id='%d' properties='%s'>\n",
+      self->priv->nbelements, properties);
+  g_free (properties);
+  g_hash_table_insert (self->priv->element_id, group,
+      GINT_TO_POINTER (self->priv->nbelements));
+  self->priv->nbelements++;
+
+  for (tmp = GES_CONTAINER_CHILDREN (group); tmp; tmp = tmp->next) {
+    gint id = GPOINTER_TO_INT (g_hash_table_lookup (self->priv->element_id,
+            tmp->data));
+
+    g_string_append_printf (str, "          <child id='%d' name='%s'/>\n", id,
+        GES_TIMELINE_ELEMENT_NAME (tmp->data));
+  }
+  g_string_append (str, "        </group>\n");
+}
+
+static void
+_save_groups (GESXmlFormatter * self, GString * str, GESTimeline * timeline)
+{
+  GList *tmp;
+  GList *seen_groups = NULL;
+
+  g_string_append (str, "      <groups>\n");
+  for (tmp = timeline_get_groups (timeline); tmp; tmp = tmp->next) {
+    _save_group (self, str, &seen_groups, tmp->data);
+  }
+  g_string_append (str, "      </groups>\n");
+}
 
 static inline void
-_save_timeline (GString * str, GESTimeline * timeline)
+_save_timeline (GESXmlFormatter * self, GString * str, GESTimeline * timeline)
 {
   gchar *properties = NULL, *metas = NULL;
 
@@ -1036,8 +1275,9 @@ _save_timeline (GString * str, GESTimeline * timeline)
       g_markup_printf_escaped
       ("    <timeline properties='%s' metadatas='%s'>\n", properties, metas));
 
-  _save_tracks (str, timeline);
-  _save_layers (str, timeline);
+  _save_tracks (self, str, timeline);
+  _save_layers (self, str, timeline);
+  _save_groups (self, str, timeline);
 
   g_string_append (str, "    </timeline>\n");
 
@@ -1046,8 +1286,8 @@ _save_timeline (GString * str, GESTimeline * timeline)
 }
 
 static void
-_save_stream_profiles (GString * str, GstEncodingProfile * sprof,
-    const gchar * profilename, guint id)
+_save_stream_profiles (GESXmlFormatter * self, GString * str,
+    GstEncodingProfile * sprof, const gchar * profilename, guint id)
 {
   gchar *tmpc;
   GstCaps *tmpcaps;
@@ -1060,6 +1300,12 @@ _save_stream_profiles (GString * str, GstEncodingProfile * sprof,
           gst_encoding_profile_get_type_nick (sprof),
           gst_encoding_profile_get_presence (sprof)));
 
+  if (!gst_encoding_profile_is_enabled (sprof)) {
+    append_escaped (str, g_strdup ("enabled='0' "));
+
+    self->priv->min_version = MAX (self->priv->min_version, 2);
+  }
+
   tmpcaps = gst_encoding_profile_get_format (sprof);
   if (tmpcaps) {
     tmpc = gst_caps_to_string (tmpcaps);
@@ -1107,7 +1353,8 @@ _save_stream_profiles (GString * str, GstEncodingProfile * sprof,
 }
 
 static inline void
-_save_encoding_profiles (GString * str, GESProject * project)
+_save_encoding_profiles (GESXmlFormatter * self, GString * str,
+    GESProject * project)
 {
   GstCaps *profformat;
   const gchar *profname, *profdesc, *profpreset, *proftype, *profpresetname;
@@ -1155,7 +1402,7 @@ _save_encoding_profiles (GString * str, GESProject * project)
       for (tmp2 = gst_encoding_container_profile_get_profiles (container_prof);
           tmp2; tmp2 = tmp2->next, i++) {
         GstEncodingProfile *sprof = (GstEncodingProfile *) tmp2->data;
-        _save_stream_profiles (str, sprof, profname, i);
+        _save_stream_profiles (self, str, sprof, profname, i);
       }
     }
     append_escaped (str,
@@ -1169,16 +1416,17 @@ _save (GESFormatter * formatter, GESTimeline * timeline, GError ** error)
   GString *str;
   GESProject *project;
 
+  gchar *projstr = NULL, *version;
   gchar *properties = NULL, *metas = NULL;
   GESXmlFormatterPrivate *priv;
 
 
   priv = _GET_PRIV (formatter);
+
+  priv->min_version = 1;
   project = formatter->project;
   str = priv->str = g_string_new (NULL);
 
-  g_string_append_printf (str, "<ges version='%i.%i'>\n", API_VERSION,
-      MINOR_VERSION);
   properties = _serialize_properties (G_OBJECT (project), NULL);
   metas = ges_meta_container_metas_to_string (GES_META_CONTAINER (project));
   append_escaped (str,
@@ -1188,16 +1436,32 @@ _save (GESFormatter * formatter, GESTimeline * timeline, GError ** error)
   g_free (metas);
 
   g_string_append (str, "    <encoding-profiles>\n");
-  _save_encoding_profiles (str, project);
+  _save_encoding_profiles (GES_XML_FORMATTER (formatter), str, project);
   g_string_append (str, "    </encoding-profiles>\n");
 
   g_string_append (str, "    <ressources>\n");
   _save_assets (str, project);
   g_string_append (str, "    </ressources>\n");
 
-  _save_timeline (str, timeline);
+  _save_timeline (GES_XML_FORMATTER (formatter), str, timeline);
   g_string_append (str, "</project>\n</ges>");
 
+  projstr = g_strdup_printf ("<ges version='%i.%i'>\n", API_VERSION,
+      priv->min_version);
+  g_string_prepend (str, projstr);
+  g_free (projstr);
+
+  ges_meta_container_set_int (GES_META_CONTAINER (project),
+      GES_META_FORMAT_VERSION, priv->min_version);
+
+  version = g_strdup_printf ("%d.%d", API_VERSION,
+      GES_XML_FORMATTER (formatter)->priv->min_version);
+
+  ges_meta_container_set_string (GES_META_CONTAINER (project),
+      GES_META_FORMAT_VERSION, version);
+
+  g_free (version);
+
   priv->str = NULL;
 
   return str;
@@ -1227,6 +1491,17 @@ ges_xml_formatter_init (GESXmlFormatter * self)
   GESXmlFormatterPrivate *priv = _GET_PRIV (self);
 
   priv->project_opened = FALSE;
+  priv->element_id = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+  self->priv = priv;
+  self->priv->min_version = 1;
+}
+
+static void
+_dispose (GObject * object)
+{
+  g_clear_pointer (&GES_XML_FORMATTER (object)->priv->element_id,
+      (GDestroyNotify) g_hash_table_unref);
 }
 
 static void
@@ -1240,6 +1515,7 @@ ges_xml_formatter_class_init (GESXmlFormatterClass * self_class)
   g_type_class_add_private (self_class, sizeof (GESXmlFormatterPrivate));
   object_class->get_property = _get_property;
   object_class->set_property = _set_property;
+  object_class->dispose = _dispose;
 
   basexmlformatter_class->content_parser.start_element = _parse_element_start;
   basexmlformatter_class->content_parser.end_element = _parse_element_end;
