@@ -76,51 +76,49 @@ _weak_notify_cb (GstFramePositionner * pos, GObject * old)
 }
 
 static void
-gst_frame_positionner_update_properties (GstFramePositionner * pos)
+gst_frame_positionner_update_properties (GstFramePositionner * pos,
+    gint old_track_width, gint old_track_height)
 {
   GstCaps *caps;
-  gint final_width;
-  gint final_height;
 
   if (pos->capsfilter == NULL)
     return;
 
-  final_width = (pos->width > 0) ? pos->width : pos->track_width;
-  final_height = (pos->height > 0) ? pos->height : pos->track_height;
-
-  if (final_width == 0 && final_height == 0)
-    caps = gst_caps_new_empty_simple ("video/x-raw");
-  else if (final_width == 0)
-    caps =
-        gst_caps_new_simple ("video/x-raw", "height", G_TYPE_INT, final_height,
-        NULL);
-  else if (final_height == 0)
-    caps =
-        gst_caps_new_simple ("video/x-raw", "width", G_TYPE_INT, final_width,
-        NULL);
-  else
+  if (pos->track_width && pos->track_height) {
     caps =
         gst_caps_new_simple ("video/x-raw", "width", G_TYPE_INT,
-        final_width, "height", G_TYPE_INT, final_height, NULL);
-
-  GST_DEBUG_OBJECT (pos, "updated size to : %d %d", final_width, final_height);
+        pos->track_width, "height", G_TYPE_INT, pos->track_height, NULL);
+  } else {
+    caps = gst_caps_new_empty_simple ("video/x-raw");
+  }
 
   if (pos->fps_n != -1)
     gst_caps_set_simple (caps, "framerate", GST_TYPE_FRACTION, pos->fps_n,
         pos->fps_d, NULL);
 
+  if (old_track_width && pos->width == old_track_width &&
+      old_track_width && pos->width == old_track_width) {
+
+    GST_DEBUG_OBJECT (pos, "FOLLOWING track size width old_track: %d -- pos: %d"
+        " || height, old_track %d -- pos: %d",
+        old_track_width, pos->width, old_track_height, pos->height);
+
+    pos->width = pos->track_width;
+    pos->height = pos->track_height;
+  }
+
   GST_DEBUG_OBJECT (pos, "setting caps : %s", gst_caps_to_string (caps));
 
   g_object_set (pos->capsfilter, "caps", caps, NULL);
 
-  g_object_notify (G_OBJECT (pos), "width");
-  g_object_notify (G_OBJECT (pos), "height");
+  gst_caps_unref (caps);
 }
 
 static void
 sync_properties_from_caps (GstFramePositionner * pos, GstCaps * caps)
 {
   gint width, height;
+  gint old_track_width, old_track_height;
 
   width = height = 0;
 
@@ -137,14 +135,17 @@ sync_properties_from_caps (GstFramePositionner * pos, GstCaps * caps)
       pos->fps_n = -1;
   }
 
+  old_track_width = pos->track_width;
+  old_track_height = pos->track_height;
+
   pos->track_width = width;
   pos->track_height = height;
 
-  GST_DEBUG_OBJECT (pos, "syncing size from caps : %d %d", width, height);
   GST_DEBUG_OBJECT (pos, "syncing framerate from caps : %d/%d", pos->fps_n,
       pos->fps_d);
 
-  gst_frame_positionner_update_properties (pos);
+  gst_frame_positionner_update_properties (pos, old_track_width,
+      old_track_height);
 }
 
 static void
@@ -196,6 +197,9 @@ ges_frame_positionner_set_source_and_filter (GstFramePositionner * pos,
   pos->track_source = trksrc;
   pos->capsfilter = capsfilter;
   pos->current_track = ges_track_element_get_track (trksrc);
+
+  g_object_add_weak_pointer (G_OBJECT (pos->track_source),
+      ((gpointer *) & pos->track_source));
   g_object_weak_ref (G_OBJECT (pos->current_track),
       (GWeakNotify) _weak_notify_cb, pos);
 
@@ -285,7 +289,7 @@ gst_frame_positionner_class_init (GstFramePositionnerClass * klass)
    */
   g_object_class_install_property (gobject_class, PROP_ZORDER,
       g_param_spec_uint ("zorder", "zorder", "z order of the stream",
-          0, 10000, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, 0, G_PARAM_READWRITE));
 
   /**
    * gesframepositionner:width:
@@ -295,7 +299,7 @@ gst_frame_positionner_class_init (GstFramePositionnerClass * klass)
    */
   g_object_class_install_property (gobject_class, PROP_WIDTH,
       g_param_spec_int ("width", "width", "width of the source",
-          0, MAX_PIXELS, 0, G_PARAM_READWRITE));
+          0, MAX_PIXELS, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
 
   /**
    * gesframepositionner:height:
@@ -305,7 +309,7 @@ gst_frame_positionner_class_init (GstFramePositionnerClass * klass)
    */
   g_object_class_install_property (gobject_class, PROP_HEIGHT,
       g_param_spec_int ("height", "height", "height of the source",
-          0, MAX_PIXELS, 0, G_PARAM_READWRITE));
+          0, MAX_PIXELS, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
 
   gst_element_class_set_static_metadata (GST_ELEMENT_CLASS (klass),
       "frame positionner", "Metadata",
@@ -354,11 +358,11 @@ gst_frame_positionner_set_property (GObject * object, guint property_id,
       break;
     case PROP_WIDTH:
       framepositionner->width = g_value_get_int (value);
-      gst_frame_positionner_update_properties (framepositionner);
+      gst_frame_positionner_update_properties (framepositionner, 0, 0);
       break;
     case PROP_HEIGHT:
       framepositionner->height = g_value_get_int (value);
-      gst_frame_positionner_update_properties (framepositionner);
+      gst_frame_positionner_update_properties (framepositionner, 0, 0);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -449,6 +453,8 @@ gst_frame_positionner_meta_transform (GstBuffer * dest, GstMeta * meta,
     dmeta->alpha = smeta->alpha;
     dmeta->posx = smeta->posx;
     dmeta->posy = smeta->posy;
+    dmeta->width = smeta->width;
+    dmeta->height = smeta->height;
     dmeta->zorder = smeta->zorder;
   }
 
@@ -474,6 +480,8 @@ gst_frame_positionner_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
   meta->alpha = framepositionner->alpha;
   meta->posx = framepositionner->posx;
   meta->posy = framepositionner->posy;
+  meta->width = framepositionner->width;
+  meta->height = framepositionner->height;
   meta->zorder = framepositionner->zorder;
   GST_OBJECT_UNLOCK (framepositionner);
 
